/* Vypocet 3D souradnic projektivniho
 * nebo metrickeho modelu
 * Autor: Beata Turonova
 */
#ifndef _POINTS_H
#define _POINTS_H

extern double cx,cy,cz,maxZ;   //prumerne hodnoty 3D bodu a maximalni hodnota z souradnice
extern double **points2D;      //souradnice bodu 2D
extern double **points3D;	   //souradnice 3D bodu

/**
* Najde minimalni hodnotu obsazenou ve vektoru
*
* @param w nesetrideny vektor vlasntich cisel
* @param n velikost vektoru
* @return c index prvku s minimalni hodnotou ve vektoru w
*/
int find_min(double *w, int n);

/**
* Najde maximalni hodnotu obsazenou ve vektoru
*
* @param w nesetrideny vektor vlasntich cisel
* @param n velikost vektoru
* @return c index prvku s maximalni hodnotou ve vektoru w
*/
int find_max(double *w, int n);

/**
* Usporada rozklad na vlastni cisla
*
* Vektor vlastnich cisel bude setriden
* sestupne a podle toho budou prehozeny
* sloupce v maticich rozkladu.
* Vsechny matice jsou velikosti 3x3
* a vektory jsou velikosti 3!!!
*
* @param S puvodni prvni matice SVD rozkladu
* @param w nesetrideny vektor vlasntich cisel
* @param V puvodni druha matice SVD rozkladu
* @param U setridena matice S
* @param Vt setridena a transponovana matice V
* @param D setridena diagonalni matice s vlastnimi cisly
*/
void order_svd(double **S, double *w, double **V, double **U, double **Vt, double **D);

/**
* Provadi transformaci souradnic
*
* Presune je tak, abz jejich stred byl
* v pocatku souradnicoveho systemu.
* Zmeni jejich velikost tak, aby prumerna
* vzdalenost od pocatku byla rovna druhe
* odmocnine.
*
* @param fm matice souradnic
* @param T1 prvni transformacni matice
* @param T2 druha transformacni matice
*/
int Scaling(double **fm,double **T1, double **T2T);

/**
* Upravi matici F tak, aby odpovidala Frobeniove norme
*
* @param Fr nenormalizovana esenncialni matice
* @param F esencialni matice po normalizaci
*/
void Frobenius(double **Fr, double **F);

/**
* Upravi matici E tak, aby odpovidala Frobeniove norme
*
* @param Er nenormalizovana esenncialni matice
* @param E esencialni matice po normalizaci
*/
void FrobeniusEssential(double **Er, double **E);

/**
* Spocita fundamentalni matici F
*
* Sestavi rovnici Af=0 a vyresi ji metodou
* nejmensich ctvercu.
*
* @param fm matice se shodnymi body
* @param F fundamentalni matice
*/
void Solving_Projective(double **fm,double **F);

/**
* Spocita esencialni matici E
*
* Nejdrive pomoci kalibracnich dat normalizuje
* 2D souradnice a pak metodou nejmensich ctvercu
* vzresi rovnici x2^T E x1=0.
* Pokud je control v compute3D() nula,
* neni tato funkce volana!
*
* @param fm matice se shodnymi body
* @param E esencialni matice
*/
void Solving_Metric(double **fm,double **E);

/**
* Spocita matici P2 pro projektivni model
*
* @param F fundamentalni matice
* @param P2 pro ulozeni druhe matice fotoaparatu
*/
void projectiveP2(double **F, double **P2);

/**
* Urci, zda nalezena P2 vyhovuje
*
* Zkusi pomoci P2 vypocitat 3D souradnici
* nehomogenni metodou a na zaklade toho
* se rozhodne, zda P2 vyhovuje
*
* @param P2 pro ulozeni druhe matice fotoaparatu
* @return   0 - matice vyhovuje
*           1 - matice nevyhovuje
*/
int testP2(double **P2);

/**
* Spocita matici P2 pro metricky model
*
* @param F fundamentalni matice
* @param P2 pro ulozeni druhe matice fotoaparatu
* @param control udava, zda je matice F fundamentali nebo esencialni
*/
void metricP2(double **F, double **P2,int control);

/**
* Optimalizuje souradnice bodu
*
* Optimalizuje souradnice 2D bodu.
* Obsahuje hledani korenu polynomu
* stupne 6 - vyuziva funkci roots.
*
* @param p matice se shodnymi 2D body
* @param F fundamentalni matice
* @param opt matice pro ulozeni optimalizovanych bodu
*/
void optimal(double **p, double **F, double **opt);

/**
* Spocita 3D souradnice homogenni metodou
*
* Vyuziva rozkladu na vlastni cisla.
* Spocita hodnoty globalnich promennych cx,cy,cz
* a maxZ.
* Neni v tomto programu volana!
*
* @param P1 prvni matice fotoaparatu
* @param P2 druha matice fotoaparatu
* @param p matice se shodnymi 2D body
*/
void homogen_method(double **P1, double **P2, double **p);

/**
* Spocita 3D souradnice nehomogenni metodou
*
* Vyuziva rozkladu na vlastni cisla.
* Spocita hodnoty globalnich promennych cx,cy,cz
* a maxZ.
*
* @param P1 prvni matice fotoaparatu
* @param P2 druha matice fotoaparatu
* @param p matice se shodnymi 2D body
*/
void inhomogen_method(double **P1, double **P2, double **p);

/**
* Vola prislusne funkce pro vypocet 3D souradnic
*
* @param p matice se shodnymi body
* @param number pocet bodu
* @param K1temp prvni kalibracni matice
* @param K2temp druha kalibracni matice
* @param metric typ modelu: 0-projektivni;1-metricky
* @return   0 - vse probehlo v poradku
*           1 - doslo k chybe
*/
int compute3D(double **p, int number, double **K1temp, double **K2temp, int metric);

#endif
