/* (C) Copr. 1986-92 Numerical Recipes in C, Second Edition */
#ifndef _ROOTS_H
#define _ROOTS_H

#include <math.h>

#define EPSS 1.0e-7
#define MR 8
#define MT 10
#define MAXIT (MT*MR)
#define EPS 2.0e-6
#define MAXM 100

/*struktura komplexnich cisel*/
typedef struct FCOMPLEX {
	double r,i;
}fcomplex;

/**
* Secte dve komplexni cisla
*
* @param a prvni komplexni cislo
* @param b druhe komplexni cislo
* @return soucet dvou komplexnich cisel
*/
fcomplex Cadd(fcomplex a, fcomplex b)
{
	fcomplex c;
	c.r=a.r+b.r;
	c.i=a.i+b.i;
	return c;
}

/**
* Vynasobi dve komplexni cisla
*
* @param a prvni komplexni cislo
* @param b druhe komplexni cislo
* @return nosobek dvou komplexnich cisel
*/
fcomplex Cmul(fcomplex a, fcomplex b)
{
	fcomplex c;
	c.r=a.r*b.r-a.i*b.i;
	c.i=a.i*b.r+a.r*b.i;
	return c;
}

/**
* Spocita absolutni hodnotu komplexniho cisla
*
* @param z komplexni cislo
* @return absolutni hodnota komplexniho cisla
*/
double Cabs(fcomplex z)
{
	double x,y,ans,temp;

	x=fabs(z.r);
	y=fabs(z.i);
	if (x == 0.0)
		ans=y;
	else if (y == 0.0)
		ans=x;
	else if (x > y)
	{
		temp=y/x;
		ans=x*sqrt(1.0+temp*temp);
	}
	else
	{
		temp=x/y;
		ans=y*sqrt(1.0+temp*temp);
	}
	return ans;
}

/**
* Vynasobi komplexni cislo realnym
*
* @param x realne cislo
* @param a komplexni cislo
* @return nasobek komplexniho a realneho cisla
*/
fcomplex RCmul(double x, fcomplex a)
{
	fcomplex c;
	c.r=x*a.r;
	c.i=x*a.i;
	return c;
}

/**
* Spocita odmocninu z komplexniho cisla
*
* @param z komplexni cislo
* @return odmocnina z komplexniho cisla
*/
fcomplex Csqrt(fcomplex z)
{
	fcomplex c;
	double x,y,w,r;
	if ((z.r == 0.0) && (z.i == 0.0))
	{
		c.r=0.0;
		c.i=0.0;
		return c;
	}
	else
	{
		x=fabs(z.r);
		y=fabs(z.i);
		if (x >= y)
		{
			r=y/x;
			w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
		}
		else
		{
			r=x/y;
			w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
		}

		if (z.r >= 0.0)
		{
			c.r=w;
			c.i=z.i/(2.0*w);
		}
		else
		{
			c.i=(z.i >= 0) ? w : -w;
			c.r=z.i/(2.0*c.i);
		}
		return c;
	}
}

/**
* Odecte dve komplexni cisla
*
* @param a prvni komplexni cislo
* @param b druhe komplexni cislo
* @return rozdil komplexnich cisel
*/
fcomplex Csub(fcomplex a, fcomplex b)
{
	fcomplex c;
	c.r=a.r-b.r;
	c.i=a.i-b.i;
	return c;
}

/**
* Vypocita komplexne sdruzene cislo
*
* @param z prvni komplexni cislo
* @return komplexne sdruzene cislo k z
*/
fcomplex Conjg(fcomplex z)
{
	fcomplex c;
	c.r=z.r;
	c.i = -z.i;
	return c;
}

/**
* Vytvori komplexni cislo
*
* @param re realna cast komplexniho cisla
* @param im imaginarni cast komplexniho cisla
* @return komplexni cislo
*/
fcomplex Complex(double re, double im)
{
	fcomplex c;
	c.r=re;
	c.i=im;
	return c;
}

/**
* Vydeli dve komplexni cisla
*
* @param a prvni komplexni cislo
* @param b druhe komplexni cislo
* @return podil dvou komplexnich cisel
*/
fcomplex Cdiv(fcomplex a, fcomplex b)
{
	fcomplex c;
	double r,den;

	if (fabs(b.r) >= fabs(b.i))
	{
		r=b.i/b.r;
		den=b.r+r*b.i;
		c.r=(a.r+r*a.i)/den;
		c.i=(a.i-r*a.r)/den;
	}
	else
	{
		r=b.r/b.i;
		den=b.i+r*b.r;
		c.r=(a.r*r+a.i)/den;
		c.i=(a.i*r-a.r)/den;
	}
	return c;
}

/**
* Najde jeden koren polynomu
*
* @param a koeficienty polynomu
* @param m stupen polynomu
* @param x vychozi hodnota, pro kterou bude funkce hledat koren
* @param its pocet iteraci
*/
void laguer(fcomplex a[], int m, fcomplex *x, int *its)
{
	int iter,j;
	double abx,abp,abm,err;
	fcomplex dx,x1,b,d,f,g,h,sq,gp,gm,g2;
	static double frac[MR+1] = {0.0,0.5,0.25,0.75,0.13,0.38,0.62,0.88,1.0};

	for (iter=1;iter<=MAXIT;iter++)
	{
		*its=iter;
		b=a[m];
		err=Cabs(b);
		d=f=Complex(0.0,0.0);
		abx=Cabs(*x);

		for (j=m-1;j>=0;j--)
		{
			f=Cadd(Cmul(*x,f),d);
			d=Cadd(Cmul(*x,d),b);
			b=Cadd(Cmul(*x,b),a[j]);
			err=Cabs(b)+abx*err;
		}

		err *= EPSS;

		if (Cabs(b) <= err) return;

		g=Cdiv(d,b);
		g2=Cmul(g,g);
		h=Csub(g2,RCmul(2.0,Cdiv(f,b)));
		sq=Csqrt(RCmul((double) (m-1),Csub(RCmul((double) m,h),g2)));
		gp=Cadd(g,sq);
		gm=Csub(g,sq);
		abp=Cabs(gp);
		abm=Cabs(gm);

		if (abp < abm) gp=gm;

		dx=((ld_max(abp,abm) > 0.0 ? Cdiv(Complex((double) m,0.0),gp)
				: RCmul(exp(log(1+abx)),Complex(cos((double)iter),sin((double)iter)))));
		x1=Csub(*x,dx);

		if (x->r == x1.r && x->i == x1.i) return;

		if (iter % MT) *x=x1;
		else *x=Csub(*x,RCmul(frac[iter/MT],dx));
	}

	return;
}

/**
* Najde koreny daneho polynomu
*
* @param a koeficienty polynomu
* @param m stupen polynomu
* @param roots pole na ulozeni nalezenych korenu polynomu
* @param polish 0 - pokud ma byt pouzito vylepseni
*               1 - pokud ne
*/
void zroots(fcomplex a[], int m, fcomplex roots[], int polish)
{
	void laguer(fcomplex a[], int m, fcomplex *x, int *its);
	int i,its,j,jj;
	fcomplex x,b,c,ad[MAXM];

	for (j=0;j<=m;j++)
		ad[j]=a[j];
	for (j=m;j>=1;j--)
	{
		x=Complex(0.0,0.0);
		laguer(ad,j,&x,&its);

		if (fabs(x.i) <= 2.0*EPS*fabs(x.r))
			x.i=0.0;

		roots[j]=x;
		b=ad[j];

		for (jj=j-1;jj>=0;jj--)
		{
			c=ad[jj];
			ad[jj]=b;
			b=Cadd(Cmul(x,b),c);
		}
	}
	if (polish)
		for (j=1;j<=m;j++)
			laguer(a,m,&roots[j],&its);

	for (j=2;j<=m;j++)
	{
		x=roots[j];
		for (i=j-1;i>=1;i--)
		{
			if (roots[i].r <= x.r) break;
			roots[i+1]=roots[i];
		}

		roots[i+1]=x;
	}
}

#endif
