/* Vypocet trojuhelnikove site z dane mnoziny bodu
 * pomoci metody vyuzivajici vnitrnich vlastnosti teto mnoziny
 * Autor: Beata Turonova
 */
#include "surface.h"
#include <math.h>
#include <string.h>
#include <gtk/gtk.h>
#include "matrix.h"
#include "triangle.h"
#include "outputs.h"

double **X;             //pozmenene 3D souradnice
int **pointTable;       //tabulka statusu bodu - 0 -aktivni, 1 - pevny
double **MinMax;        //obsahuje max a min delku hran prilehlych bodu
list<edge> inner;       //seznam vnitrnich hran
list<triangle> meshT;   //trojuhelnikova sit

/**
* Tridici funkce pro quicksort
*
* Tridi vzdalenosti vzestupne
*
* @param v1 prvni porovnavana vzdalenost
* @param v2 druha porovnavana vzdalenost
*/
int wmlCompare(const void *v1,const void *v2)
{
    wml *p1,*p2;
    p1 = (wml*)v1;
    p2 = (wml*)v2;

	if (p1->dist < p2->dist)
		return(-1);
    else if (p1->dist> p2->dist)
		return(1);
    else
		return(0);
}

/**
* Normalizace vektoru
*
* Spocte velikost daneho vektoru a vydeli touto
* hodnoutou jeho jednotlive slozky
*
* @param vector vektor, ktery ma byt normalizovan
*/
void normVector(double *vector)
{
	double dist;
	double norm[3];

	dist=sqrt(vector[0]*vector[0]+vector[1]*vector[1]+vector[2]*vector[2]);    //velikost vektoru
	norm[0]=vector[0]/dist;
	norm[1]=vector[1]/dist;
	norm[2]=vector[2]/dist;

	vector[0]=norm[0];
	vector[1]=norm[1];
	vector[2]=norm[2];
}

/**
* Vytvori stenu
*
* @param P bod udavajici stenu
* @param test udava, ktery poloprostor ma byt testovan na pritomnost bodu
* @param normal normala udavajici rovinu
* @return newF nova stena
*/
face createFace(xyz P, int test, double *normal)
{
	face newF;

	newF.P.x=P.x;
	newF.P.y=P.y;
	newF.P.z=P.z;
	newF.test=test;
	newF.normal[0]=normal[0];
	newF.normal[1]=normal[1];
	newF.normal[2]=normal[2];

	return newF;
}

/**
* Vytvori novou hranu
*
* Pro hranu spocita jeji stred, teziste a normalu
* trojuhelniku daneho body ida, idb, a o
*
* @param ida identifikator prvniho bodu udavajiciho hranu
* @param idb identifikator druheho bodu udavajiciho hranu
* @param o identifikator protejsiho bodu hrany
* @param flag status hrany - aktivni nebo hranicni
* @return newE nova hrana
*/
edge createEdge(int ida, int idb, int o, int flag)
{
	edge newE;
	double normal[3];
	double vec1[3], vec2[3];

	vec1[0]=X[ida][0]-X[o][0];
	vec1[1]=X[ida][1]-X[o][1];
	vec1[2]=X[ida][2]-X[o][2];

	vec2[0]=X[idb][0]-X[o][0];
	vec2[1]=X[idb][1]-X[o][1];
	vec2[2]=X[idb][2]-X[o][2];

	crossProduct(vec1,vec2,normal);

    /*stred hrany*/
	newE.center[0]=(X[ida][0]+X[idb][0])/2;
	newE.center[1]=(X[ida][1]+X[idb][1])/2;
	newE.center[2]=(X[ida][2]+X[idb][2])/2;

	/*normala*/
	newE.normal[0]=normal[0];
	newE.normal[1]=normal[1];
	newE.normal[2]=normal[2];

	newE.ida=ida;
	newE.idb=idb;
	newE.o=o;   //protejsi vrchol hrany

	/*teziste trojuhelniku ida,idb,o*/
	newE.P[0]=(X[ida][0]+X[idb][0]+X[o][0])/3;
	newE.P[1]=(X[ida][1]+X[idb][1]+X[o][1])/3;
	newE.P[2]=(X[ida][2]+X[idb][2]+X[o][2])/3;

	newE.flag=flag;

	return newE;
}

/**
* Spocte vzdalenost bodu A od bodu B
*
* @param A identifikator prvniho bodu
* @param B identifikator druheho bodu
* @return d druha mocnina vzdalenosti
*/
double distance2(int A, int B)
{
	double a,b,c;
	double o,p,q;
	double d;

	a=X[A][0];
	b=X[A][1];
	c=X[A][2];

	o=X[B][0];
	p=X[B][1];
	q=X[B][2];

	d=((a-o)*(a-o) + (b-p)*(b-p) + (c-q)*(c-q));

	return d;
}

/**
* Urci projekci bodu pt
* na primku prochazejici stredy podstav valce cyl
*
* @param cyl valec
* @param pt bod, ktery testujeme
* @return projekce bodu pt na primku
*/
double projection(cylinder cyl, xyz pt)
{
	xyz	pr;

	Sub(cyl.cc2,cyl.cc1,pr);

	return (innerProduct2(pt,pr) - innerProduct2(cyl.cc1,pr))
	      / innerProduct2(pr,pr);
}

/**
* Urci bod na primce prochazejici stredy podstav valce cyl
*
* @param cyl valec
* @param t parametr primky
* @return pt bod lezici na dane primce
*/
xyz pointCL(cylinder cyl, double t)
{
	xyz	par;
	xyz pt;

	Sub(cyl.cc2,cyl.cc1,par);

	pt.x = cyl.cc1.x + t * par.x;
	pt.y = cyl.cc1.y + t * par.y;
	pt.z = cyl.cc1.z + t * par.z;

	return pt;
}


/**
* Urci vzdalenost bodu od primky prchazejici stredy podstav valce
*
* @param cyl valec
* @param t parametr primky
* @param pt bod,jehoz vydalenost od primky chceme zjistit
* @return vzdalenost bodu od primky
*/
double distanceCL(cylinder cyl, double tmin, xyz pt)
{
	xyz	ptL, d;

	ptL=pointCL(cyl, tmin);

	Sub(ptL,pt,d);

	return sqrt(innerProduct2(d,d));
}


/**
* Spocte obsah trojuhelniku daneho body a, b,c
*
* Obsah je roven jedne polovine z odmocniny
* souctu tri determinantu o velikostech 3x3,
* jejichz sloupce jsou tvoreny souradnicemi
* bodu
*
* @param a identifikator prvniho bodu
* @param b identifikator druheho bodu
* @param c identifikator tretiho bodu
* @return d obsah trojuhelniku
*/
double areaTriangle(int a, int b, int c)
{
	double m00, m01, m02;
	double m10, m11, m12;
	double m20, m21, m22;
	double d;
	double d1, d2, d3;

	/*prvni determinant*/
	m00=X[a][0];
	m01=X[a][1];
	m02=1.0;

	m10=X[b][0];
	m11=X[b][1];
	m12=1.0;

	m20=X[c][0];
	m21=X[c][1];
	m22=1.0;

	d1=m00*m11*m22+m10*m21*m02+m01*m12*m20-m02*m11*m20-m01*m10*m22-m00*m12*m21;
	d1=d1*d1;

	/*druhy determinant*/
	m00=X[a][1];
	m01=X[a][2];

	m10=X[b][1];
	m11=X[b][2];

	m20=X[c][1];
	m21=X[c][2];

	d2=m00*m11*m22+m10*m21*m02+m01*m12*m20-m02*m11*m20-m01*m10*m22-m00*m12*m21;
	d2=d2*d2;

    /*treti determinant*/
	m00=X[a][2];
	m01=X[a][0];

	m10=X[b][2];
	m11=X[b][0];

	m20=X[c][2];
	m21=X[c][0];

	d3=m00*m11*m22+m10*m21*m02+m01*m12*m20-m02*m11*m20-m01*m10*m22-m00*m12*m21;
	d3=d3*d3;

	d=sqrt(d1+d2+d3)/2;

	return d;
}

/**
* Aktualizuje tabulky MinMax a pointTable
*
* Po pridani noveho trojuhelniku upravi hodnoty
* v tabulce MinMax, popr. upravi statusy bodu a,b
* v pointTable
*
* @param a identifikator prvniho bodu hrany
* @param b identifikator druheho bodu hrany
* @param skip 0, pokud hrana nebyla zpracovana
*             1, pokud byla
*/
void changeTables(int a, int b, int skip)
{
	double d1;  //vzdalenost
	list<edge>::iterator it;
	int ca, cb;
	int stopa, stopb;

	if(skip==0)         //dosud nezpracovana hrana
	{
		pointTable[a][1]=pointTable[a][1]+1;    //navysi pocet hran prilehlych k bodu a
		pointTable[b][1]=pointTable[b][1]+1;    //navysi pocet hran prilehlych k bodu b

		d1=sqrt(distance2(a,b));
		if(d1<MinMax[a][0])
			MinMax[a][0]=d1;
		if(d1>MinMax[a][1])
			MinMax[a][1]=d1;

		if (pointTable[b][1]==1)					//novy bod
		{
			d1=sqrt(distance2(b,a));
			MinMax[b][0]=d1;
			MinMax[b][1]=d1;
		}
		else
		{
			d1=sqrt(distance2(b,a));
			if(d1<MinMax[b][0])
				MinMax[b][0]=d1;
			if(d1>MinMax[b][1])
				MinMax[b][1]=d1;
		}
	}

	ca=cb=0;
	stopa=0;
	stopb=0;

	for (it=inner.begin();it!=inner.end();it++)		//projde seznam vnitrnich hran
	{
		if(((*it).ida==a) || ((*it).idb==a))
		{
			ca++;
			if(pointTable[a][1]==ca)				//vsechny prilehle hrany bodu a jsou vnitrni
			{
				pointTable[a][0]=1;					//bod a je pevny
				stopa=1;
				if(stopb==1)
					break;
			}
		}
		if(((*it).ida==b) || ((*it).idb==b))
		{
			cb++;
			if(pointTable[b][1]==cb)				//vsechny prilehle hrany bodu b jsou vnitrni
			{
				pointTable[b][0]=1;					//bod b je pevny
				stopb=1;
				if(stopa==1)
					break;
			}
		}
	}
}

/**
* Vytvori novy trojuhelnik dany body a, b, c
*
* @param a identifikator prvniho bodu
* @param b identifikator druheho bodu
* @param c identifikator tretiho bodu
* @return triangle novy trojuhelnik
*/
triangle createTriangle(int a, int b, int c)
{
	triangle t;

	t.a.x=X[a][0];
	t.a.y=X[a][1];
	t.a.z=X[a][2];

	t.b.x=X[b][0];
	t.b.y=X[b][1];
	t.b.z=X[b][2];

	t.c.x=X[c][0];
	t.c.y=X[c][1];
	t.c.z=X[c][2];

	t.ida=a;
	t.idb=b;
	t.idc=c;

	return t;
}

/**
* Nalezne trojuhelnikovou sit
*
* Pouzit algoritmus vyuzivajici vnitrnich
* vlasntosti vstupni mnoziny bodu
*
* @param points matice se souradnicema 3D bodu
* @param n pocet bodu
* @return   0 pokud vypocet probehl v poradku
*           1 pokud byl vypocet ukoncen kvuli chybe
*/
int mesh(double **points, int n)
{
	double zmax;	        //hranice oblasti
	double dmin;            //minimalni vzdalenost
	double dist2;
	edge aE;                //aktivni hrana
	int i;
	list<int> pIn;          //seznam bodu uvnitr oblasti vlivu
	int index;
	cylinder cyl;           //valec, ve kterem hedame prvni trojuhelnik
	xyz	 pt;
	double tmin, dist;
	int third;
	list<edge> active;      //seznam aktvnich a hranicnich hran
	list<face> F;           //seznam sten udavajicich ovlast vlivu
	xyz P;                  //bod udavajici stenu
	double a[3];            //bod a aktivni hrany
	double b[3];            //bod b aktivni hrany
	double normal[3];       //normala
	double pp[3];           //vektorovy soucin normal
	int inside;             //0-bod je uvnitr oblasti vlivu, 1-neni
	double d1, d2;          //vzdalenosti
	int countP;             //pocet bodu uvnitr oblasti vlivlu
	double k1, k2;
	double A1, A2;          //obsahy trojuhleniku
	double btw, btw2, btw3; //mezisoucty
	double L1, L2, L3;
	double s;               //SUD
	list<edge>::iterator it;
	list<int>::iterator iter;
	list<triangle>::iterator itT;
	list<face>::iterator itF;
	int in1, in2,in3;       //indikatory pritomnosti hran v seznamech
	edge e;
	list<triangle> testT;   //seznam trojuhelniku urcenzch k testovani na pruseciky
	double u,v, uv;         //velikosti vektoru
	int first;              //0-prvni nalezena hodnota, 1-dalsi
	double minAngle, angle; //uhly
	int found;              //0-torjuhelnik nenalezen, 1-nalezen
	int pass;
	triangle t;
	int inter;              //0-trojuhelniky se neprotinaji, 1-prusecik existuje
	list<edge>::iterator er1,er2, er4, er5;
	int test;
	int help;           //bod s max. zetovou souradnici
	int mult;           //nasobime jeji hodnotou vstupni body
	wml *sa;            //pole s nejmensima vzdalenostma bodu v oblasti vlivu od aktivni hrany
	int j;
	vector<listT> adj;  //seznam trojuhelniku prilehlych bodum
	triangle tri;
	int ts;
	int skip;           //testovani,0-testovat,1-netestovat
	int t1;

	adj.resize(n);

	X=matrix(n,3);
	mult=1000;
	zmax = points[0][2]*mult;
	help=0;

	/*upraveni 3D bodu a nalezeni toho s max. z-souradnici*/
	for (i=0;i<n;i++)
	{
        X[i][0]=points[i][0]*mult;
        X[i][1]=points[i][1]*mult;
        X[i][2]=points[i][2]*mult;
        if (X[i][2] > zmax)
        {
			zmax = X[i][2];
			help=i;
		}
	}

	/* nalezeni prvniho trojuhelniku */

	if(help==1)
	{
		dmin=sqrt(distance2(help,2));
		index=2;
	}
	else
	{
		dmin=sqrt(distance2(help,1));
		index=1;
	}

	for(i=0;i<n;i++)        //najde bod s minimalni vzdalenosti od bodu help
	{
		if(i!=help && i!=1)
		{
			dist2=sqrt(distance2(help,i));
			if (dist2<dmin)
			{
				dmin=dist2;
				index=i;
			}
		}
	}

	countP=0;

	/*valec se stredy podstav v help a index*/
	cyl.cc1.x = X[help][0];
	cyl.cc1.y = X[help][1];
	cyl.cc1.z = X[help][2];

	cyl.cc2.x = X[index][0];
	cyl.cc2.y = X[index][1];
	cyl.cc2.z = X[index][2];

	cyl.radius = dmin/2;

	/*najde body obsazene ve valci*/
	while(countP<1)
	{
		for(i=0;i<n;i++)
		{
			if(index!=i && help!=i)
			{
				pt.x = X[i][0];
				pt.y = X[i][1];
				pt.z = X[i][2];

				tmin = projection(cyl, pt);
				dist = distanceCL(cyl, tmin, pt);

				if ((tmin > 0) && (tmin < 1) && (dist < cyl.radius))
				{
					pIn.push_back(i);
					countP++;
				}
			}
		}

		cyl.cc2.x = cyl.cc2.x+(dmin/2);
		cyl.cc2.y = cyl.cc2.y+(dmin/2);
		cyl.cc2.z = cyl.cc2.z+(dmin/2);

		cyl.cc1.x = cyl.cc1.x-(dmin/2);
		cyl.cc1.y = cyl.cc1.y-(dmin/2);
		cyl.cc1.z = cyl.cc1.z-(dmin/2);

		cyl.radius = 2*cyl.radius;
	}

	/*z bodu ve valci vybere ten nejmin vzdaleny od index a help*/
	dmin=sqrt(distance2(help,(*pIn.begin())))+sqrt(distance2(index,(*pIn.begin())));
	third=(*pIn.begin());
	pIn.pop_front();
    for(iter=pIn.begin();iter!=pIn.end();iter++)
	{
		dist2=sqrt(distance2(help,(*iter)))+sqrt(distance2(index,(*iter)));
		if(dist2<dmin )
		{
			third=(*iter);
			dmin=dist2;
		}
	}

	pIn.clear();

	pointTable=intMatrix(n,2);
	MinMax=matrix(n,2);

	/*nastavi hodnoty v tabulce pointTable*/
	for (i=0; i<n;i++)
	{
		if ((i==index) || (i==third) || (i==help))
		{
			pointTable[i][0]=0;		//0 aktivni, 1 pevny
			pointTable[i][1]=2;		//pocet prilehlych hran
		}
		else
		{
			pointTable[i][0]=0;		//0 aktivni, 1 pevny
			pointTable[i][1]=0;		//pocet prilehlych hran
		}
	}

    /*upravi smer normaly prvniho trojuhelniku, pokud bude potreba*/
	e=createEdge(help,index,third,0);
	if (e.normal[2]<0.0)
	{
		e.normal[0]=-e.normal[0];
		e.normal[1]=-e.normal[1];
		e.normal[2]=-e.normal[2];
	}
	active.push_back(e);

	e=createEdge(index,third,help,0);
	if (e.normal[2]<0.0)
	{
		e.normal[0]=-e.normal[0];
		e.normal[1]=-e.normal[1];
		e.normal[2]=-e.normal[2];
	}
	active.push_back(e);

	e=createEdge(third,help,index,0);
	if (e.normal[2]<0.0)
	{
		e.normal[0]=-e.normal[0];
		e.normal[1]=-e.normal[1];
		e.normal[2]=-e.normal[2];
	}
	active.push_back(e);

    /*spocita min a max velikosti hran prilehlych bodum help,index,third*/
	d1=sqrt(distance2(help,index));
	d2=sqrt(distance2(help,third));
	if(d1<d2)
	{
		MinMax[help][0]=d1;
		MinMax[help][1]=d2;
	}
	else
	{
		MinMax[help][0]=d2;
		MinMax[help][1]=d1;
	}

	d1=sqrt(distance2(index,help));
	d2=sqrt(distance2(index,third));
	if(d1<d2)
	{
		MinMax[index][0]=d1;
		MinMax[index][1]=d2;
	}
	else
	{
		MinMax[index][0]=d2;
		MinMax[index][1]=d1;
	}

	d1=sqrt(distance2(third,help));
	d2=sqrt(distance2(third,index));
	if(d1<d2)
	{
		MinMax[third][0]=d1;
		MinMax[third][1]=d2;
	}
	else
	{
		MinMax[third][0]=d2;
		MinMax[third][1]=d1;
	}

    /*vlozi prvni trojuhelnik do seznamu se siti*/
	tri=createTriangle(help,index,third);
	adj[help].t.push_back(tri);
	adj[index].t.push_back(tri);
	adj[third].t.push_back(tri);
	meshT.push_back(tri);

	/*postupne vybirani novych trojuhelniku - jeden cyklus=jeden trojuhelnik*/
	while(!active.empty())
	{
		pass=0;

		/*vezme prvni aktivni hranu v seznamu*/
		for(it=active.begin();it!=active.end();it++)
		{
			if((*it).flag==0)		//je aktivni
			{
				aE=(*it);
				active.erase(it);
				pass=1;
				break;
			}
		}

		if(pass==0) break; //nezbyla zadna aktivni hrana

		pass=0;

		d1=MinMax[aE.ida][1]/MinMax[aE.ida][0];     //Lmax/Lmin u bodu a
		d2=MinMax[aE.idb][1]/MinMax[aE.idb][0];     //Lmax/Lmin u bodu b
		if(d1<d2)
			{s=d2*((MinMax[aE.ida][0]+MinMax[aE.idb][0])/2);}
		else
			{s=d1*((MinMax[aE.ida][0]+MinMax[aE.idb][0])/2);}

		a[0]=X[aE.ida][0];
		a[1]=X[aE.ida][1];
		a[2]=X[aE.ida][2];

		b[0]=X[aE.idb][0];
		b[1]=X[aE.idb][1];
		b[2]=X[aE.idb][2];

		/*Urci stenu S1*/
		normal[0]=aE.normal[0];
		normal[1]=aE.normal[1];
		normal[2]=aE.normal[2];
		P.x=aE.center[0]+s*aE.normal[0];
		P.y=aE.center[1]+s*aE.normal[1];
		P.z=aE.center[2]+s*aE.normal[2];
		testSpace2(normal,P,aE.ida,test);       //zjisti orientaci normaly
		testSpace2(normal,P,aE.idb,test);       //zjisti orientaci normaly
		F.push_back(createFace(P,test,normal)); //vlozi stenu do seznamu

		/*Urci stenu S2*/
		normal[0]=-aE.normal[0];
		normal[1]=-aE.normal[1];
		normal[2]=-aE.normal[2];
		P.x=aE.center[0]-s*aE.normal[0];
		P.y=aE.center[1]-s*aE.normal[1];
		P.z=aE.center[2]-s*aE.normal[2];
		testSpace2(normal,P,aE.ida,test);
		testSpace2(normal,P,aE.idb,test);
		F.push_back(createFace(P,test,normal));

		/*Urci stenu S3*/
		pp[0]=a[0]-aE.P[0];
		pp[1]=a[1]-aE.P[1];
		pp[2]=a[2]-aE.P[2];
		crossProduct(aE.normal,pp,normal);
		normVector(normal);
		P.x=a[0];
		P.y=a[1];
		P.z=a[2];
		testSpace2(normal,P,aE.ida,test);
		testSpace2(normal,P,aE.idb,test);
		F.push_back(createFace(P,test,normal));

		/*najde hrany, ktere jsou na stejne strane steny S3 jako je hrana ida,idb*/
		/*a vybere z nich tu, ktera svira s hranou ida,idb nejmensi uhel*/
		first=0;
		sizeVector(aE.ida,aE.idb,u);
		for(it=active.begin();it!=active.end();it++)
		{
			if ((*it).ida==aE.ida)
			{
				testSpace2(normal,P,(*it).idb,t1);
				if(t1==test)
				{
					sizeVector(aE.ida,(*it).idb,v);
					innerProduct3(aE.ida, aE.idb, aE.ida, (*it).idb,uv);
					if(first==0)
					{
						minAngle=acos(uv/(u*v));
						index=(*it).idb;
						first=1;
					}
					else
					{
						angle=acos(uv/(u*v));
						if(angle<minAngle)
						{
							minAngle=angle;
							index=(*it).idb;
						}
					}
				}
			}
			else if ((*it).idb==aE.ida)
			{
				testSpace2(normal,P,(*it).ida,t1);
				if(test==t1)
				{
					sizeVector(aE.ida,(*it).ida,v);
					innerProduct3(aE.ida, aE.idb, aE.ida, (*it).ida,uv);
					if(first==0)
					{
						minAngle=acos(uv/(u*v));
						index=(*it).ida;
						first=1;
					}
					else
					{
						angle=acos(uv/(u*v));
						if(angle<minAngle)
						{
							minAngle=angle;
							index=(*it).ida;
						}
					}
				}
			}
		}

		/*V pripade potreby urci stenu S6*/
		if(first==1)
		{
			pp[0]=X[index][0]-a[0];
			pp[1]=X[index][1]-a[1];
			pp[2]=X[index][2]-a[2];
			crossProduct(aE.normal,pp,normal);
			P.x=X[index][0];
			P.y=X[index][1];
			P.z=X[index][2];
			testSpace2(normal,P,aE.ida,test);
			testSpace2(normal,P,aE.idb,test);
			F.push_back(createFace(P,test,normal));
		}

		/*Urci stenu S4*/
		pp[0]=b[0]-aE.P[0];
		pp[1]=b[1]-aE.P[1];
		pp[2]=b[2]-aE.P[2];
		crossProduct(aE.normal,pp,normal);
		normVector(normal);
		P.x=b[0];
		P.y=b[1];
		P.z=b[2];
		testSpace2(normal,P,aE.idb,test);
		testSpace2(normal,P,aE.ida,test);
		F.push_back(createFace(P,test,normal));

		/*najde hrany, ktere jsou na stejne strane steny S4 jako je hrana ida,idb*/
		/*a vybere z nich tu, ktera svira s hranou ida,idb nejmensi uhel*/
		first=0;
		for(it=active.begin();it!=active.end();it++)
		{
			if ((*it).ida==aE.idb)
			{
				testSpace2(normal,P,(*it).idb,t1);
				if(test==t1)
				{
					sizeVector(aE.idb,(*it).idb,v);
					innerProduct3(aE.idb, aE.ida, aE.idb, (*it).idb,uv);
					if(first==0)
					{
						minAngle=acos(uv/(u*v));
						index=(*it).idb;
						first=1;
					}
					else
					{
						angle=acos(uv/(u*v));
						if(angle<minAngle)
						{
							minAngle=angle;
							index=(*it).idb;
						}
					}
				}
			}
			else if ((*it).idb==aE.idb)
			{
				testSpace2(normal,P,(*it).ida,t1);
				if(test==t1)
				{
					sizeVector(aE.idb,(*it).ida,v);
					innerProduct3(aE.idb, aE.ida, aE.idb, (*it).ida,uv);
					if(first==0)
					{
						minAngle=acos(uv/(u*v));
						index=(*it).ida;
						first=1;
					}
					else
					{
						angle=acos(uv/(u*v));
						if(angle<minAngle)
						{
							minAngle=angle;
							index=(*it).ida;
						}
					}
				}
			}
		}

        /*V pripade potreby urci stenu S7*/
		if(first==1)
		{
			pp[0]=X[index][0]-b[0];
			pp[1]=X[index][1]-b[1];
			pp[2]=X[index][2]-b[2];
			crossProduct(aE.normal,pp,normal);
			P.x=X[index][0];
			P.y=X[index][1];
			P.z=X[index][2];
			testSpace2(normal,P,aE.idb,test);
			testSpace2(normal,P,aE.ida,test);
			F.push_back(createFace(P,test,normal));
		}

		/*Urci stenu S5*/
		pp[0]=b[0]-a[0];
		pp[1]=b[1]-a[1];
		pp[2]=b[2]-a[2];
		crossProduct(pp,aE.normal,normal);
		P.x=aE.center[0]+s*normal[0];
		P.y=aE.center[1]+s*normal[1];
		P.z=aE.center[2]+s*normal[2];
		testSpace2(normal,P,aE.ida,test);
		testSpace2(normal,P,aE.idb,test);
		F.push_back(createFace(P,test,normal));


		countP=0;
		/*testovani jednotlivych zbyvajicich bodu*/
		for(i=0;i<n;i++)
		{
			if ((i!=aE.ida) && (i!=aE.idb) && (i!=aE.o))
			{
				inside=0;
				/*otestuje zda je bod uvnitr oblasti vlivu*/
				for(itF=F.begin();itF!=F.end();itF++)
				{
					testSpace2((*itF).normal,(*itF).P,i,ts);
					if(ts!=(*itF).test && ts!=0.0)
					{
						inside=1; //neni uvnitr
						break;
					}
				}
				if (inside==0)
				{
					pIn.push_back(i);
					++countP;
				}
			}
		}

		F.clear();

		if (countP==0)  //boundary
		{
			active.push_back(createEdge(aE.ida,aE.idb,aE.o,1));
		}
		else
		{
			found=0;

			i=pIn.size();
			sa=(wml*)malloc(i*sizeof(wml));
			if (!sa)
			{
                my_error(6);
                return 1;
            }

			A2=areaTriangle(aE.ida,aE.idb,aE.o);            //obsah aktualniho trojuhelniku
			btw=(distance2(aE.ida, aE.o)+					//ik
				distance2(aE.idb, aE.o)-					//jk
				distance2(aE.ida, aE.idb))/A2;				//ij
			L3=distance2(aE.ida, aE.idb);
			j=0;
			/*spocita vazene vzdalenosti vsech bodu v oblasti vlivu od aktivni hrany*/
			for(iter=pIn.begin();iter!=pIn.end();iter++)
			{
				if((pointTable[(*iter)][0]==0) && ((*iter)!=aE.ida) && ((*iter)!=aE.idb) && ((*iter)!=aE.o))
				{
					A1=areaTriangle(aE.ida,aE.idb,(*iter));     //obsah noveho trojuhelniku
					distance(aE.ida,(*iter),L1);			    //im
					distance(aE.idb,(*iter),L2);			    //jm
					btw2=(L1+L2-L3)/A1;
					k1=btw+btw2;
					k2=2*btw2;
					btw3=k1*L3+k2*L1+k2*L2;
					sa[j].dist=btw3;
					sa[j].id=(*iter);
					j++;
				}
				/*vyber trojuhelniku urcenych k testovani*/
				if(((*iter)!=aE.ida) && ((*iter)!=aE.idb) && ((*iter)!=aE.o))
				{
					for(itT=adj[(*iter)].t.begin(); itT!=adj[(*iter)].t.end(); itT++)
					{
						insertTriangle(testT,(*itT));
					}
				}
			}
			pIn.clear();
			qsort(sa,j,sizeof(wml),wmlCompare); //seradi body podle vzdalenosti vzestupne
			i=0;
			while(i<j && found==0)
			{
				index=sa[i].id;
				i++;

				in3=0;
				/*test, zda novy trojuhelnik neobsahuje hrany, ktere jsou vnitrni*/
				for (it=inner.begin();it!=inner.end();it++)
				{
					if( ((*it).ida==aE.ida && (*it).idb==index) ||
						((*it).idb==aE.ida && (*it).ida==index) ||
						((*it).ida==aE.idb && (*it).idb==index) ||
						((*it).idb==aE.idb && (*it).ida==index))
					{
						in3=1; //je v seznamu - trojuhelnik nemuze byt vytvoren
						break;
					}
				}

				if(in3!=1)
				{
					in1=0;	//neni v seznamu
					in2=0;	//neni v seznamu

					/*test, zda hrany trojuhelnika nejsou ve fronte*/
					for (it=active.begin();it!=active.end();it++)
					{
						if(((*it).ida==aE.ida && (*it).idb==index) ||
							((*it).idb==aE.ida && (*it).ida==index))
						{
							in1=1; //je v seznamu
							er1=it;
							if(in2==1)
								break;
						}
						if(((*it).ida==aE.idb && (*it).idb==index) ||
							((*it).idb==aE.idb && (*it).ida==index))
						{
							in2=1; //je v seznamu
							er2=it;
							if(in1==1)
								break;
						}
					}

					t.ida=aE.ida;
					t.idb=aE.idb;
					t.idc=index;
					t.a.x=X[aE.ida][0];
					t.a.y=X[aE.ida][1];
					t.a.z=X[aE.ida][2];
					t.b.x=X[aE.idb][0];
					t.b.y=X[aE.idb][1];
					t.b.z=X[aE.idb][2];
					t.c.x=X[index][0];
					t.c.y=X[index][1];
					t.c.z=X[index][2];
					inter=0;
					skip=0;
					/*testovani pruseciku novehu trojuhelniku s jiz existujicima*/
					for(itT=testT.begin(); itT!=testT.end(); itT++)
					{
						if (in1==1)
						{
							if(((*itT).ida==index) || ((*itT).idb==index) || ((*itT).idc==index))
							{
									inter=0;
									skip=1;
							}
							if(    ((*itT).ida==aE.ida && (*itT).idb==index)
								|| ((*itT).ida==aE.ida && (*itT).idc==index)
								|| ((*itT).idb==aE.ida && (*itT).idc==index)
								|| ((*itT).ida==index && (*itT).idb==aE.ida)
								|| ((*itT).ida==index && (*itT).idc==aE.ida)
								|| ((*itT).idb==index && (*itT).idc==aE.ida)
							  )
							{
								inter=0;
								skip=1;
							}
						}
						if (in2==1)
						{
							if(((*itT).ida==index) || ((*itT).idb==index) || ((*itT).idc==index))
							{
									inter=0;
									skip=1;
							}
							if(    ((*itT).ida==aE.idb && (*itT).idb==index)
								|| ((*itT).ida==aE.idb && (*itT).idc==index)
								|| ((*itT).idb==aE.idb && (*itT).idc==index)
								|| ((*itT).ida==index && (*itT).idb==aE.idb)
								|| ((*itT).ida==index && (*itT).idc==aE.idb)
								|| ((*itT).idb==index && (*itT).idc==aE.idb)
							  )
							{
								inter=0;
								skip=1;
							}
						}
						if(skip==0)
						{
							inter=triangleTest(t,(*itT));
							if(inter==1)
							{
								break;
							}
						}
					}

					/*vlozi trojuhelnik do site, hrany do fronty,opravi tabulky*/
					if(inter==0)
					{
						inner.push_back(createEdge(aE.ida,aE.idb,aE.o,2));
						if(in1==1)
						{
							inner.push_back((*er1));
							changeTables((*er1).ida,(*er1).idb,1);
							active.erase(er1);
						}
						if(in2==1)
						{
							inner.push_back((*er2));
							changeTables((*er2).ida,(*er2).idb,1);
							active.erase(er2);
						}
						if(in1!=1)
						{
							e=createEdge(aE.ida,index,aE.idb,0);
							active.push_back(e);
							changeTables(aE.ida, index,0);
						}
						if(in2!=1)
						{
							e=createEdge(aE.idb,index,aE.ida,0);
							active.push_back(e);
							changeTables(aE.idb, index,0);
						}
						found=1;
						tri=createTriangle(aE.ida, aE.idb, index);
						adj[aE.ida].t.push_back(tri);
						adj[aE.idb].t.push_back(tri);
						adj[index].t.push_back(tri);
						meshT.push_back(tri);
					}
				} //end of if(in3!=1)
			} //end of while
			if(found==0)
			{
				active.push_back(createEdge(aE.ida,aE.idb,aE.o,1));
				testT.clear();
				free(sa);
			}
			else
			{
				free(sa);
				testT.clear();
			}
		}
	}

	free_matrix(MinMax);
	free_matrix(pointTable);
	free_matrix(X);
	return 0;
}
