/*
 * numerics/linalg/svdcmp.cc
 *
 * Singular value decomposition.
 *
 * Copyright (c) 2003--2004 by Wolfgang Wieser (wwieser@gmx.de)
 *
 * The SVD algorithm is based on a routine by G.E. Forsythe,
 * M.A. Malcolm, and C.B. Moler which in turn is based on
 * the original algorithm of G.H. Golub and C. Reinsch.
 *
 * This file may be distributed and/or modified under the terms of the
 * GNU General Public License version 2 as published by the Free Software
 * Foundation.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef _SVDCMP_H
#define _SVDCMP_H

/**
* Vrati minimum ze dvou cisel
*
* @param a prvni cislo k porovnani
* @param b druhe cislo k porovnani
* @return minimum y cisel a,b
*/
int ld_min(int a, int b)
{
	if (b>a) return a;
	else return b;
}

/**
* Vrati maximum ze dvou cisel
*
* @param a prvni cislo k porovnani
* @param b druhe cislo k porovnani
* @return maximum y cisel a,b
*/
double ld_max(double a, double b)
{
	if (b>a) return b;
	else return a;
}

/**
* Zmeni znamenko cisla
*
* @param a cislo, jehoz znamenko chceme zmenit
* @param b kriterium, podle ktereho se o zmene rozhodne
* @return a, nebo -a v zavislosti na hodnote b
*/
double sign(double a, double b)
{
	if (b>=0)
        {return fabs(a);}
	else
        {return -fabs(a);}
}

/**
* Spocita druhou mocninu cisla
*
* @param a mocnene cislo
* @return druha mocnina cisla a
*/
double sqr(double a)
{
	if (a==0) return a;
	else return a*a;
}

/**
* Spocita odmocninu z a^2+b^2
*
* @param a prvni cislo
* @param b druhe cislo
* @return odmocnina z a^2+b^2
*/
double pythag(double a, double b)
{
	double fa,fb;
	fa=fabs(a);
	fb=fabs(b);
	if (fa > fb) return fa*sqrt(1.0+sqr(fb/fa));
	else if (fb == 0.0 ) return 0.0;
	else return (fb*sqrt(1.0+sqr(fa/fb)));
}

/**
* Spocita rozklad na vlastni cisla
*
* @param a matice, ktera je rozkladana
* @param m pocet radku matice a
* @param n pocet sloupcu matice A
* @param w vektor s vlastnimi cisly - nijak neserazen
* @param v matice s vlastnimi vektory velikosti n*n, netransponovana
*/
void svdcmp(double **a, int m, int n,double *w, double **v)
{

		int flag, i, its, j, jj, k, l=0, nm=0;
		double anorm,c,f,g,h,s,scale,x,y,z;
		double *rv1;

		rv1 = myVector(n);

		g = scale = anorm = 0.0;

		for(i=0;i<n;i++)
		{
			l = i+1;
			rv1[i] = scale*g;
			g = s = scale = 0.0;
			if(i < m)
			{
				for(k=i;k<m;k++)
                    scale = scale + fabs(a[k][i]);
				if(scale)
				{
					for(k=i;k<m;k++)
					{
						a[k][i] = a[k][i]/scale;
						s = s + a[k][i] * a[k][i];
					}

					f = a[i][i];
					g = -sign(sqrt(s),f);
					h = f * g - s;
					a[i][i] = f - g;

					for(j=l;j<n;j++)
					{
						for(s=0.0,k=i;k<m;k++)
                            s = s + a[k][i] * a[k][j];
						f = s / h;
						for(k=i;k<m;k++)
                            a[k][j] = a[k][j] + f * a[k][i];
					}

					for(k=i;k<m;k++)
                        a[k][i] = a[k][i] *scale;
				}
			}

			w[i] = scale * g;
			g = s = scale = 0.0;

			if(i < m && i != n-1)
			{
				for(k=l;k<n;k++)
                    scale = scale + fabs(a[i][k]);
				if(scale)
				{
					for(k=l;k<n;k++)
					{
						a[i][k] = a[i][k] / scale;
						s = s + a[i][k] * a[i][k];
					}

					f = a[i][l];
					g = - sign(sqrt(s),f);
					h = f * g - s;
					a[i][l] = f - g;

					for(k=l;k<n;k++) rv1[k] = a[i][k] / h;

					for(j=l;j<m;j++)
					{
						for(s=0.0,k=l;k<n;k++) s = s +a[j][k] * a[i][k];
						for(k=l;k<n;k++) a[j][k] = a[j][k] + s * rv1[k];
					}

					for(k=l;k<n;k++) a[i][k] = a[i][k] * scale;
				}
			}

			anorm = ld_max(anorm, (fabs(w[i]) + fabs(rv1[i])));

		}

		for(i=n-1;i>=0;i--)
		{
			if(i < n-1)
			{
				if(g)
				{
					for(j=l;j<n;j++)
						v[j][i] = (a[i][j] / a[i][l]) / g;

					for(j=l;j<n;j++)
					{
						for(s=0.0,k=l;k<n;k++)
                            s = s + a[i][k] * v[k][j];
						for(k=l;k<n;k++)
                            v[k][j] =v[k][j] + s * v[k][i];
					}
				}

				for(j=l;j<n;j++) v[i][j] = v[j][i] = 0.0;
			}

			v[i][i] = 1.0;
			g = rv1[i];
			l = i;
		}

		for(i=ld_min(m,n) - 1;i >= 0;i--)
		{
			l = i + 1;
			g = w[i];

			for(j=l;j<n;j++) a[i][j] = 0.0;

			if(g)
			{
				g = 1.0 / g;
				for(j=l;j<n;j++)
				{
					for(s=0.0,k=l;k<m;k++) s = s + a[k][i] * a[k][j];
					f = (s / a[i][i]) * g;
					for(k=i;k<m;k++) a[k][j] =a[k][j] + f * a[k][i];
				}

				for(j=i;j<m;j++) a[j][i] = a[j][i] *g;
			}
			else
				for(j=i;j<m;j++) a[j][i] = 0.0;

			++a[i][i];
		}

		for(k=n-1;k>=0;k--)
		{
			for(its=0;its<30;its++)
			{
				flag = 1;
				for(l=k;l>=0;l--)
				{
					nm = l-1;
					if((fabs(rv1[l]) + anorm) == anorm)
					{
						flag =  0;
						 break;
					}

					if((fabs(w[nm]) + anorm) == anorm) break;
				}

				if(flag)
				{
					c = 0.0;
					s = 1.0;
					for(i=l;i<=k;i++)
					{
						f = s * rv1[i];
						rv1[i] = c * rv1[i];
						if((fabs(f) + anorm) == anorm) break;
						g = w[i];
						h = pythag(f,g);
						w[i] = h;
						h = 1.0 / h;
						c = g * h;
						s = -f * h;

						for(j=0;j<m;j++)
						{
							y = a[j][nm];
							z = a[j][i];
							a[j][nm] = y * c + z * s;
							a[j][i] = z * c - y * s;
						}
					}
				}

				z = w[k];

				if(l == k)
				{
					if(z < 0.0)
					{
						w[k] = -z;
						for(j=0;j<n;j++) v[j][k] = -v[j][k];
					}
					break;
				}

				if (its == 30) return;

				x = w[l];
				nm = k-1;
				y = w[nm];
				g = rv1[nm];
				h = rv1[k];
				f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0 * h * y);
				g = pythag(f,1.0);
				f = ((x - z) * (x + z) + h * ((y / (f + sign(g,f))) - h)) / x;
				c = s = 1.0;

				for(j=l;j<=nm;j++)
				{
					i = j+1;
					g = rv1[i];
					y = w[i];
					h = s * g;
					g = c * g;
					z = pythag(f,h);
					rv1[j] = z;
					c = f/z;
					s = h/z;
					f = x * c + g * s;
					g = g * c - x * s;
					h = y * s;
					y = y *c;

					for(jj=0;jj<n;jj++)
					{
						x = v[jj][j];
						z = v[jj][i];
						v[jj][j] = x * c + z * s;
						v[jj][i] = z * c - x * s;
					}

					z = pythag(f,h);
					w[j] = z;

					if(z)
					{
						z = 1.0 / z;
						c = f * z;
						s = h * z;
					}

					f = c * g + s * y;
					x = c * y - s * g;

					for(jj=0;jj < m;jj++)
					{
						y = a[jj][j];
						z = a[jj][i];
						a[jj][j] = y * c + z * s;
						a[jj][i] = z * c - y * s;
					}
				}

				rv1[l] = 0.0;
				rv1[k] = f;
				w[k] = x;
			}
		}

		free(rv1);
}
#endif
